@@ -555,7 +555,7 @@ class LocalGitBranch(GitBranch):
 
     def _get_head(self):
         try:
-            return self.repository._git.ref(self.ref or "HEAD")
+            return self.repository._git.refs[self.ref or "HEAD"]
         except KeyError:
             return None
 
@@ -1053,7 +1053,7 @@ class InterToGitBranch(branch.GenericInterBranch):
                 return True
             # FIXME: If one side only has the git sha available and the other only
             # has the bzr revid, then this will cause us to show a tag as updated
-            # that hasn't actually been updated. 
+            # that hasn't actually been updated.
             return False
         # FIXME: Check for diverged branches
         for ref, (git_sha, revid) in new_refs.iteritems():
