@@ -17,6 +17,7 @@
 
 """A grouping of Exceptions for bzr-git"""
 
+from __future__ import absolute_import
 
 from dulwich import errors as git_errors
 
@@ -27,24 +28,14 @@ class BzrGitError(bzr_errors.BzrError):
     """The base-level exception for bzr-git errors."""
 
 
-class GitCommandError(BzrGitError):
-    """Raised when spawning 'git' does not return normally."""
-
-    _fmt = 'Command failed (%(returncode)s): command %(command)s\n%(stderr)s'
-
-    def __init__(self, command, returncode, stderr):
-        self.command = command
-        self.returncode = returncode
-        self.stderr = stderr
-
-
 class NoSuchRef(BzrGitError):
     """Raised when a ref can not be found."""
 
-    _fmt = "The ref %(ref)s was not found."
+    _fmt = "The ref %(ref)s was not found in the repository at %(location)s."
 
-    def __init__(self, ref, present_refs=None):
+    def __init__(self, ref, location, present_refs=None):
         self.ref = ref
+        self.location = location
         self.present_refs = present_refs
 
 
@@ -60,5 +51,23 @@ class NoPushSupport(bzr_errors.BzrError):
     _fmt = "Push is not yet supported for bzr-git. Try dpush instead."
 
 
-class GitSmartRemoteNotSupported(bzr_errors.BzrError):
+class GitSmartRemoteNotSupported(bzr_errors.UnsupportedOperation):
     _fmt = "This operation is not supported by the Git smart server protocol."
+
+
+class UnknownCommitExtra(bzr_errors.BzrError):
+    _fmt = "Unknown extra fields in %(object)r: %(fields)r."
+
+    def __init__(self, object, fields):
+        bzr_errors.BzrError.__init__(self)
+        self.object = object
+        self.fields = ",".join(fields)
+
+
+class UnknownMercurialCommitExtra(bzr_errors.BzrError):
+    _fmt = "Unknown mercurial extra fields in %(object)r: %(fields)r."
+
+    def __init__(self, object, fields):
+        bzr_errors.BzrError.__init__(self)
+        self.object = object
+        self.fields = ",".join(fields)
