@@ -14,6 +14,11 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
+from __future__ import absolute_import
+
+from dulwich.errors import (
+    NotCommitError,
+    )
 from dulwich.objects import (
     Commit,
     Tag,
@@ -23,6 +28,7 @@ from dulwich.objects import (
     ZERO_SHA,
     )
 from dulwich.object_store import (
+    ObjectStoreGraphWalker,
     tree_lookup_path,
     )
 from dulwich.walk import Walker
@@ -42,10 +48,8 @@ from bzrlib import (
     )
 from bzrlib.errors import (
     BzrError,
-    NoSuchId,
     )
 from bzrlib.inventory import (
-    Inventory,
     InventoryDirectory,
     InventoryFile,
     InventoryLink,
@@ -68,9 +72,6 @@ from bzrlib.versionedfile import (
     ChunkedContentFactory,
     )
 
-from bzrlib.plugins.git.errors import (
-    NotCommitError,
-    )
 from bzrlib.plugins.git.mapping import (
     DEFAULT_FILE_MODE,
     mode_is_executable,
@@ -95,9 +96,9 @@ from bzrlib.plugins.git.repository import (
     )
 
 
-def import_git_blob(texts, mapping, path, name, (base_hexsha, hexsha), 
-        base_inv, parent_id, revision_id,
-        parent_invs, lookup_object, (base_mode, mode), store_updater,
+def import_git_blob(texts, mapping, path, name, (base_hexsha, hexsha),
+        base_bzr_tree, parent_id, revision_id,
+        parent_bzr_trees, lookup_object, (base_mode, mode), store_updater,
         lookup_file_id):
     """Import a git blob object into a bzr repository.
 
@@ -120,13 +121,16 @@ def import_git_blob(texts, mapping, path, name, (base_hexsha, hexsha),
     if ie.kind == "file":
         ie.executable = mode_is_executable(mode)
     if base_hexsha == hexsha and mode_kind(base_mode) == mode_kind(mode):
-        base_ie = base_inv[base_inv.path2id(path)]
-        ie.text_size = base_ie.text_size
-        ie.text_sha1 = base_ie.text_sha1
+        base_file_id = base_bzr_tree.path2id(path)
+        base_exec = base_bzr_tree.is_executable(base_file_id, path)
         if ie.kind == "symlink":
-            ie.symlink_target = base_ie.symlink_target
-        if ie.executable == base_ie.executable:
-            ie.revision = base_ie.revision
+            ie.symlink_target = base_bzr_tree.get_symlink_target(
+                base_file_id, path)
+        else:
+            ie.text_size = base_bzr_tree.get_file_size(base_file_id)
+            ie.text_sha1 = base_bzr_tree.get_file_sha1(base_file_id, path)
+        if ie.kind == "symlink" or ie.executable == base_exec:
+            ie.revision = base_bzr_tree.get_file_revision(base_file_id, path)
         else:
             blob = lookup_object(hexsha)
     else:
@@ -139,18 +143,19 @@ def import_git_blob(texts, mapping, path, name, (base_hexsha, hexsha),
             ie.text_sha1 = osutils.sha_strings(blob.chunked)
     # Check what revision we should store
     parent_keys = []
-    for pinv in parent_invs:
+    for ptree in parent_bzr_trees:
         try:
-            pie = pinv[file_id]
-        except NoSuchId:
+            pkind = ptree.kind(file_id)
+        except errors.NoSuchId:
             continue
-        if (pie.text_sha1 == ie.text_sha1 and
-            pie.executable == ie.executable and
-            pie.symlink_target == ie.symlink_target):
+        if (pkind == ie.kind and
+            ((pkind == "symlink" and ptree.get_symlink_target(file_id) == ie.symlink_target) or
+             (pkind == "file" and ptree.get_file_sha1(file_id) == ie.text_sha1 and
+                ptree.is_executable(file_id) == ie.executable))):
             # found a revision in one of the parents to use
-            ie.revision = pie.revision
+            ie.revision = ptree.get_file_revision(file_id)
             break
-        parent_key = (file_id, pie.revision)
+        parent_key = (file_id, ptree.get_file_revision(file_id))
         if not parent_key in parent_keys:
             parent_keys.append(parent_key)
     if ie.revision is None:
@@ -168,26 +173,25 @@ def import_git_blob(texts, mapping, path, name, (base_hexsha, hexsha),
     if base_hexsha is not None:
         old_path = path.decode("utf-8") # Renames are not supported yet
         if stat.S_ISDIR(base_mode):
-            invdelta.extend(remove_disappeared_children(base_inv, old_path,
+            invdelta.extend(remove_disappeared_children(base_bzr_tree, old_path,
                 lookup_object(base_hexsha), [], lookup_object))
     else:
         old_path = None
     new_path = path.decode("utf-8")
     invdelta.append((old_path, new_path, file_id, ie))
     if base_hexsha != hexsha:
-        store_updater.add_object(blob, ie, path)
+        store_updater.add_object(blob, (ie.file_id, ie.revision), path)
     return invdelta
 
 
 class SubmodulesRequireSubtrees(BzrError):
-    _fmt = ("The repository you are fetching from contains submodules. "
-            "To continue, upgrade your Bazaar repository to a format that "
-            "supports nested trees, such as 'development-subtree'.")
+    _fmt = ("The repository you are fetching from contains submodules, "
+            "which are not yet supported.")
     internal = False
 
 
 def import_git_submodule(texts, mapping, path, name, (base_hexsha, hexsha),
-    base_inv, parent_id, revision_id, parent_invs, lookup_object,
+    base_bzr_tree, parent_id, revision_id, parent_bzr_trees, lookup_object,
     (base_mode, mode), store_updater, lookup_file_id):
     """Import a git submodule."""
     if base_hexsha == hexsha and base_mode == mode:
@@ -199,7 +203,7 @@ def import_git_submodule(texts, mapping, path, name, (base_hexsha, hexsha),
     if base_hexsha is not None:
         old_path = path.decode("utf-8") # Renames are not supported yet
         if stat.S_ISDIR(base_mode):
-            invdelta.extend(remove_disappeared_children(base_inv, old_path,
+            invdelta.extend(remove_disappeared_children(base_bzr_tree, old_path,
                 lookup_object(base_hexsha), [], lookup_object))
     else:
         old_path = None
@@ -210,11 +214,11 @@ def import_git_submodule(texts, mapping, path, name, (base_hexsha, hexsha),
     return invdelta, {}
 
 
-def remove_disappeared_children(base_inv, path, base_tree, existing_children,
+def remove_disappeared_children(base_bzr_tree, path, base_tree, existing_children,
         lookup_object):
     """Generate an inventory delta for removed children.
 
-    :param base_inv: Base inventory against which to generate the 
+    :param base_bzr_tree: Base bzr tree against which to generate the
         inventory delta.
     :param path: Path to process (unicode)
     :param base_tree: Git Tree base object
@@ -228,17 +232,17 @@ def remove_disappeared_children(base_inv, path, base_tree, existing_children,
         if name in existing_children:
             continue
         c_path = posixpath.join(path, name.decode("utf-8"))
-        file_id = base_inv.path2id(c_path)
+        file_id = base_bzr_tree.path2id(c_path)
         assert file_id is not None
         ret.append((c_path, None, file_id, None))
         if stat.S_ISDIR(mode):
             ret.extend(remove_disappeared_children(
-                base_inv, c_path, lookup_object(hexsha), [], lookup_object))
+                base_bzr_tree, c_path, lookup_object(hexsha), [], lookup_object))
     return ret
 
 
 def import_git_tree(texts, mapping, path, name, (base_hexsha, hexsha),
-        base_inv, parent_id, revision_id, parent_invs,
+        base_bzr_tree, parent_id, revision_id, parent_bzr_trees,
         lookup_object, (base_mode, mode), store_updater,
         lookup_file_id, allow_submodules=False):
     """Import a git tree object into a bzr repository.
@@ -247,7 +251,7 @@ def import_git_tree(texts, mapping, path, name, (base_hexsha, hexsha),
     :param path: Path in the tree (str)
     :param name: Name of the tree (str)
     :param tree: A git tree object
-    :param base_inv: Base inventory against which to return inventory delta
+    :param base_bzr_tree: Base inventory against which to return inventory delta
     :return: Inventory delta for this subtree
     """
     assert type(path) is str
@@ -289,22 +293,23 @@ def import_git_tree(texts, mapping, path, name, (base_hexsha, hexsha),
             child_base_mode = 0
         if stat.S_ISDIR(child_mode):
             subinvdelta, grandchildmodes = import_git_tree(texts, mapping,
-                child_path, name, (child_base_hexsha, child_hexsha), base_inv,
-                file_id, revision_id, parent_invs, lookup_object,
-                (child_base_mode, child_mode), store_updater, lookup_file_id,
-                allow_submodules=allow_submodules)
+                child_path, name, (child_base_hexsha, child_hexsha),
+                base_bzr_tree, file_id, revision_id, parent_bzr_trees,
+                lookup_object, (child_base_mode, child_mode), store_updater,
+                lookup_file_id, allow_submodules=allow_submodules)
         elif S_ISGITLINK(child_mode): # submodule
             if not allow_submodules:
                 raise SubmodulesRequireSubtrees()
             subinvdelta, grandchildmodes = import_git_submodule(texts, mapping,
-                child_path, name, (child_base_hexsha, child_hexsha), base_inv,
-                file_id, revision_id, parent_invs, lookup_object,
-                (child_base_mode, child_mode), store_updater, lookup_file_id)
+                child_path, name, (child_base_hexsha, child_hexsha),
+                base_bzr_tree, file_id, revision_id, parent_bzr_trees,
+                lookup_object, (child_base_mode, child_mode), store_updater,
+                lookup_file_id)
         else:
             if not mapping.is_special_file(name):
                 subinvdelta = import_git_blob(texts, mapping, child_path, name,
-                    (child_base_hexsha, child_hexsha), base_inv, file_id,
-                    revision_id, parent_invs, lookup_object,
+                    (child_base_hexsha, child_hexsha), base_bzr_tree, file_id,
+                    revision_id, parent_bzr_trees, lookup_object,
                     (child_base_mode, child_mode), store_updater, lookup_file_id)
             else:
                 subinvdelta = []
@@ -317,9 +322,9 @@ def import_git_tree(texts, mapping, path, name, (base_hexsha, hexsha),
             child_modes[child_path] = child_mode
     # Remove any children that have disappeared
     if base_tree is not None and type(base_tree) is Tree:
-        invdelta.extend(remove_disappeared_children(base_inv, old_path,
+        invdelta.extend(remove_disappeared_children(base_bzr_tree, old_path,
             base_tree, existing_children, lookup_object))
-    store_updater.add_object(tree, ie, path)
+    store_updater.add_object(tree, (file_id, ), path)
     return invdelta, child_modes
 
 
@@ -387,18 +392,18 @@ def import_git_commit(repo, mapping, head, lookup_object,
     parent_trees = trees_cache.revision_trees(rev.parent_ids)
     ensure_inventories_in_repo(repo, parent_trees)
     if parent_trees == []:
-        base_inv = Inventory(root_id=None)
+        base_bzr_tree = trees_cache.revision_tree(NULL_REVISION)
         base_tree = None
         base_mode = None
     else:
-        base_inv = parent_trees[0].inventory
+        base_bzr_tree = parent_trees[0]
         base_tree = lookup_object(o.parents[0]).tree
         base_mode = stat.S_IFDIR
     store_updater = target_git_object_retriever._get_updater(rev)
     tree_supplement = mapping.get_fileid_map(lookup_object, o.tree)
     inv_delta, unusual_modes = import_git_tree(repo.texts,
-            mapping, "", "", (base_tree, o.tree), base_inv,
-            None, rev.revision_id, [p.inventory for p in parent_trees],
+            mapping, "", "", (base_tree, o.tree), base_bzr_tree,
+            None, rev.revision_id, parent_trees,
             lookup_object, (base_mode, stat.S_IFDIR), store_updater,
             tree_supplement.lookup_file_id,
             allow_submodules=getattr(repo._format, "supports_tree_reference",
@@ -411,9 +416,15 @@ def import_git_commit(repo, mapping, head, lookup_object,
         basis_id = rev.parent_ids[0]
     except IndexError:
         basis_id = NULL_REVISION
-        base_inv = None
+        base_bzr_inventory = None
+    else:
+        try:
+            base_bzr_inventory = base_bzr_tree.root_inventory
+        except AttributeError: # bzr < 2.6
+            base_bzr_inventory = base_bzr_tree.inventory
     rev.inventory_sha1, inv = repo.add_inventory_by_delta(basis_id,
-              inv_delta, rev.revision_id, rev.parent_ids, base_inv)
+              inv_delta, rev.revision_id, rev.parent_ids,
+              base_bzr_inventory)
     ret_tree = InventoryRevisionTree(repo, inv, rev.revision_id)
     # Check verifiers
     if verifiers and roundtrip_revid is not None:
@@ -425,7 +436,7 @@ def import_git_commit(repo, mapping, head, lookup_object,
                          rev.revision_id, verifiers["testament3-sha1"])
             rev.revision_id = original_revid
             rev.inventory_sha1, inv = repo.add_inventory_by_delta(basis_id,
-              inv_delta, rev.revision_id, rev.parent_ids, base_inv)
+              inv_delta, rev.revision_id, rev.parent_ids, base_bzr_tree)
             ret_tree = InventoryRevisionTree(repo, inv, rev.revision_id)
     else:
         calculated_verifiers = {}
@@ -434,7 +445,7 @@ def import_git_commit(repo, mapping, head, lookup_object,
     trees_cache.add(ret_tree)
     repo.add_revision(rev.revision_id, rev)
     if "verify" in debug.debug_flags:
-        verify_commit_reconstruction(target_git_object_retriever, 
+        verify_commit_reconstruction(target_git_object_retriever,
             lookup_object, o, rev, ret_tree, parent_trees, mapping,
             unusual_modes, verifiers)
 
@@ -493,7 +504,7 @@ def import_git_objects(repo, mapping, object_iter,
         revision_ids = revision_ids[:limit]
     last_imported = None
     for offset in range(0, len(revision_ids), batch_size):
-        target_git_object_retriever.start_write_group() 
+        target_git_object_retriever.start_write_group()
         try:
             repo.start_write_group()
             try:
@@ -692,8 +703,9 @@ class InterRemoteGitNonGitRepository(InterGitNonGitRepository):
         store.lock_write()
         try:
             heads = self.get_target_heads()
-            graph_walker = store.get_graph_walker(
-                    [store._lookup_revision_sha1(head) for head in heads])
+            graph_walker = ObjectStoreGraphWalker(
+                [store._lookup_revision_sha1(head) for head in heads],
+                lambda sha: store[sha].parents)
             wants_recorder = DetermineWantsRecorder(determine_wants)
 
             pb = ui.ui_factory.nested_progress_bar()
