@@ -43,6 +43,7 @@ from bzrlib.revision import (
 from bzrlib.plugins.git.errors import (
     NoPushSupport,
     UnknownCommitExtra,
+    UnknownMercurialCommitExtra,
     )
 from bzrlib.plugins.git.hg import (
     format_hg_metadata,
@@ -57,6 +58,11 @@ from bzrlib.plugins.git.roundtrip import (
     )
 
 DEFAULT_FILE_MODE = stat.S_IFREG | 0644
+HG_RENAME_SOURCE = "HG:rename-source"
+HG_EXTRA = "HG:extra"
+
+# This HG extra is used to indicate the commit that this commit was based on.
+HG_EXTRA_AMEND_SOURCE = "amend_source"
 
 
 def escape_file_id(file_id):
@@ -86,6 +92,8 @@ def unescape_file_id(file_id):
 
 def fix_person_identifier(text):
     if "<" in text and ">" in text:
+        if not " <" in text and text.count("<") == 1:
+            text = text.replace("<", " <")
         return text
     return "%s <%s>" % (text, text)
 
@@ -290,7 +298,7 @@ class BzrGitMapping(foreign.VcsMapping):
             commit.author_timezone = int(rev.properties['author-timezone'])
         else:
             commit.author_timezone = commit.commit_timezone
-        commit.message = self._encode_commit_message(rev, rev.message, 
+        commit.message = self._encode_commit_message(rev, rev.message,
             encoding)
         assert type(commit.message) == str
         if metadata is not None:
@@ -307,11 +315,13 @@ class BzrGitMapping(foreign.VcsMapping):
                     metadata.properties[k] = v
         if not lossy:
             if self.roundtripping:
-                commit.message = inject_bzr_metadata(commit.message, metadata, 
+                commit.message = inject_bzr_metadata(commit.message, metadata,
                                                      encoding)
             else:
                 raise NoPushSupport()
         assert type(commit.message) == str
+        if 'git-extra' in rev.properties:
+            commit.extra.extend([l.split(' ', 1) for l in rev.properties['git-extra'].splitlines()])
         return commit
 
     def import_fileid_map(self, blob):
@@ -330,8 +340,6 @@ class BzrGitMapping(foreign.VcsMapping):
         """
         if commit is None:
             raise AssertionError("Commit object can't be None")
-        if commit.extra:
-            raise UnknownCommitExtra(commit, [item[0] for item in commit.extra])
         rev = ForeignRevision(commit.id, self,
                 self.revision_id_foreign_to_bzr(commit.id))
         rev.git_metadata = None
@@ -377,6 +385,22 @@ class BzrGitMapping(foreign.VcsMapping):
             verifiers = {}
         if rev.parent_ids is None:
             rev.parent_ids = tuple([lookup_parent_revid(p) for p in commit.parents])
+        unknown_extra_fields = []
+        extra_lines = []
+        for k, v in commit.extra:
+            if k == HG_RENAME_SOURCE:
+                extra_lines.append(k + ' ' + v + '\n')
+            elif k == HG_EXTRA:
+                hgk, hgv = v.split(':', 1)
+                if hgk not in (HG_EXTRA_AMEND_SOURCE, ):
+                    raise UnknownMercurialCommitExtra(commit, hgk)
+                extra_lines.append(k + ' ' + v + '\n')
+            else:
+                unknown_extra_fields.append(k)
+        if unknown_extra_fields:
+            raise UnknownCommitExtra(commit, unknown_extra_fields)
+        if extra_lines:
+            rev.properties['git-extra'] = ''.join(extra_lines)
         return rev, roundtrip_revid, verifiers
 
     def get_fileid_map(self, lookup_object, tree_sha):
