@@ -173,7 +173,7 @@ def _tree_to_objects(tree, parent_trees, idmap, unusual_modes,
         in empty directories. None to skip empty directories
     :return: Yields (path, object, ie) entries
     """
-    new_trees = {}
+    dirty_dirs = set()
     new_blobs = []
     shamap = {}
     try:
@@ -190,7 +190,7 @@ def _tree_to_objects(tree, parent_trees, idmap, unusual_modes,
                 pass
             else:
                 if kind == "file":
-                    if (pkind == "file" and 
+                    if (pkind == "file" and
                         ptree.get_file_sha1(file_id) == other):
                         return (file_id, ptree.get_file_revision(file_id))
                 if kind == "symlink":
@@ -219,7 +219,6 @@ def _tree_to_objects(tree, parent_trees, idmap, unusual_modes,
                         shamap[file_id] = blob.id
             if not file_id in shamap:
                 new_blobs.append((path[1], file_id))
-            new_trees[posixpath.dirname(path[1])] = parent[1]
         elif kind[1] == "symlink":
             if changed_content:
                 target = tree.get_symlink_target(file_id)
@@ -229,14 +228,11 @@ def _tree_to_objects(tree, parent_trees, idmap, unusual_modes,
                     find_unchanged_parent_ie(file_id, kind[1], target, other_parent_trees)
                 except KeyError:
                     yield path[1], blob, (file_id, tree.get_file_revision(file_id, path[1]))
-            new_trees[posixpath.dirname(path[1])] = parent[1]
         elif kind[1] not in (None, "directory"):
             raise AssertionError(kind[1])
-        if (path[0] not in (None, "") and
-            tree.has_id(parent[0]) and
-            tree.kind(parent[0]) == "directory"):
-            # Removal
-            new_trees[posixpath.dirname(path[0])] = parent[0]
+        for p in parent:
+            if p and tree.has_id(p) and tree.kind(p) == "directory":
+                dirty_dirs.add(p)
 
     # Fetch contents of the blobs that were changed
     for (path, file_id), chunks in tree.iter_files_bytes(
@@ -250,18 +246,24 @@ def _tree_to_objects(tree, parent_trees, idmap, unusual_modes,
         parent_path = posixpath.dirname(path)
         file_id = tree.path2id(parent_path)
         assert file_id is not None, "Unable to find file id for %r" % parent_path
-        new_trees[parent_path] = file_id
+        dirty_dirs.add(file_id)
+
+    try:
+        inv = tree.root_inventory
+    except AttributeError:
+        inv = tree.inventory
 
     trees = {}
-    while new_trees:
-        items = new_trees.items()
-        new_trees = {}
-        for path, file_id in items:
-            if path != "":
-                parent_path = urlutils.dirname(path)
-                parent_id = tree.path2id(parent_path)
-                new_trees[parent_path] = parent_id
-            trees[path] = file_id
+    while dirty_dirs:
+        new_dirs = set()
+        for file_id in dirty_dirs:
+            if file_id is None or not inv.has_id(file_id):
+                continue
+            trees[inv.id2path(file_id)] = file_id
+            ie = inv[file_id]
+            if ie.parent_id is not None:
+                new_dirs.add(ie.parent_id)
+        dirty_dirs = new_dirs
 
     def ie_to_hexsha(ie):
         try:
@@ -277,7 +279,7 @@ def _tree_to_objects(tree, parent_trees, idmap, unusual_modes,
                     blob.data = tree.get_file_text(ie.file_id)
                     return blob.id
             elif ie.kind == "directory":
-                # Not all cache backends store the tree information, 
+                # Not all cache backends store the tree information,
                 # calculate again from scratch
                 ret = directory_to_tree(ie.children, ie_to_hexsha,
                     unusual_modes, dummy_file_name, ie.parent_id is None)
@@ -287,11 +289,6 @@ def _tree_to_objects(tree, parent_trees, idmap, unusual_modes,
             else:
                 raise AssertionError
 
-    try:
-        inv = tree.root_inventory
-    except AttributeError:
-        inv = tree.inventory
-
     for path in sorted(trees.keys(), reverse=True):
         file_id = trees[path]
         assert tree.kind(file_id) == 'directory'
@@ -415,7 +412,7 @@ class BazaarObjectStore(BaseObjectStore):
             lossy, verifiers)
 
     def _create_fileid_map_blob(self, tree):
-        # FIXME: This can probably be a lot more efficient, 
+        # FIXME: This can probably be a lot more efficient,
         # not all files necessarily have to be processed.
         file_ids = {}
         for (path, ie) in tree.inventory.iter_entries():
@@ -586,6 +583,8 @@ class BazaarObjectStore(BaseObjectStore):
 
         :param sha: SHA1 of the git object
         """
+        if len(sha) == 20:
+            sha = sha_to_hex(sha)
         obj = self[sha]
         return (obj.type, obj.as_raw_string())
 
@@ -781,6 +780,6 @@ class BazaarObjectStore(BaseObjectStore):
                 self.repository.unlock()
         return f, commit
 
-    # The pack isn't kept around anyway, so no point 
+    # The pack isn't kept around anyway, so no point
     # in treating full packs different from thin packs
     add_pack = add_thin_pack
