@@ -16,6 +16,8 @@
 
 """Push implementation that simply prints message saying push is not supported."""
 
+from __future__ import absolute_import
+
 from dulwich.objects import ZERO_SHA
 from dulwich.walk import Walker
 
@@ -236,7 +238,7 @@ class InterToLocalGitRepository(InterToGitRepository):
         bzr_refs = {}
         refs = {}
         for k in self.target._git.refs.allkeys():
-            v = self.target._git.refs.read_ref(k)
+            v = self.target._git.refs[k]
             try:
                 for (kind, type_data) in self.source_store.lookup_git_sha(v):
                     if kind == "commit" and self.source.has_revision(type_data[0]):
@@ -283,14 +285,15 @@ class InterToLocalGitRepository(InterToGitRepository):
                     self.source_store, self.source, pb)
                 for (old_revid, git_sha) in object_generator.import_revisions(
                     todo, lossy=lossy):
-                    try:
-                        self.mapping.revision_id_bzr_to_foreign(old_revid)
-                    except errors.InvalidRevisionId:
-                        self.target_refs[self.mapping.revid_as_refname(old_revid)] = git_sha
                     if lossy:
                         new_revid = self.mapping.revision_id_foreign_to_bzr(git_sha)
                     else:
                         new_revid = old_revid
+                        try:
+                            self.mapping.revision_id_bzr_to_foreign(old_revid)
+                        except errors.InvalidRevisionId:
+                            refname = self.mapping.revid_as_refname(old_revid)
+                            self.target_refs[refname] = git_sha
                     revidmap[old_revid] = (git_sha, new_revid)
                 self.target_store.add_objects(object_generator)
                 return revidmap
