@@ -17,6 +17,8 @@
 
 """An adapter between a Git Repository and a Bazaar Branch"""
 
+from __future__ import absolute_import
+
 from bzrlib import (
     check,
     errors,
@@ -25,13 +27,11 @@ from bzrlib import (
     repository,
     revision,
     transactions,
+    ui,
     version_info as bzrlib_version,
     )
 from bzrlib.decorators import only_raises
-try:
-    from bzrlib.revisiontree import InventoryRevisionTree
-except ImportError: # bzr < 2.4
-    from bzrlib.revisiontree import RevisionTree as InventoryRevisionTree
+from bzrlib.revisiontree import InventoryRevisionTree
 from bzrlib.foreign import (
     ForeignRepository,
     )
@@ -39,9 +39,6 @@ from bzrlib.foreign import (
 from bzrlib.plugins.git.commit import (
     GitCommitBuilder,
     )
-from bzrlib.plugins.git.errors import (
-    NotCommitError,
-    )
 from bzrlib.plugins.git.filegraph import (
     GitFileLastChangeScanner,
     GitFileParentProvider,
@@ -56,9 +53,11 @@ from bzrlib.plugins.git.tree import (
     )
 
 
+from dulwich.errors import (
+    NotCommitError,
+    )
 from dulwich.objects import (
     Commit,
-    Tag,
     ZERO_SHA,
     )
 from dulwich.object_store import (
@@ -131,16 +130,9 @@ class GitRepository(ForeignRepository):
     chk_bytes = None
 
     def __init__(self, gitdir):
-        if bzrlib_version >= (2, 5):
-            control_files = None
-        else:
-            class DummyControlFiles(object):
-                def __init__(self):
-                    self._transport = gitdir.root_transport
-            control_files = DummyControlFiles()
         self._transport = gitdir.root_transport
         super(GitRepository, self).__init__(GitRepositoryFormat(),
-            gitdir, control_files)
+            gitdir, control_files=None)
         self.base = gitdir.root_transport.base
         lazy_load_optimisers()
         self._lock_mode = None
@@ -221,15 +213,11 @@ class GitRepository(ForeignRepository):
     def supports_rich_root(self):
         return True
 
-    def _warn_if_deprecated(self, branch=None): # for bzr < 2.4
-        # This class isn't deprecated
-        pass
-
     def get_mapping(self):
         return default_mapping
 
     def make_working_trees(self):
-        return not self._git.bare
+        return not self._git.get_config().get_boolean(("core", ), "bare")
 
     def revision_graph_can_have_wrong_parents(self):
         return False
@@ -333,6 +321,17 @@ class LocalGitRepository(GitRepository):
                     else:
                         raise AssertionError("file text resolved to %r" % obj)
 
+    def gather_stats(self, revid=None, committers=None):
+        """See Repository.gather_stats()."""
+        result = super(LocalGitRepository, self).gather_stats(revid, committers)
+        revs = []
+        for sha in self._git.object_store:
+            o = self._git.object_store[sha]
+            if o.type_name == "commit":
+                revs.append(o.id)
+        result['revisions'] = len(revs)
+        return result
+
     def _iter_revision_ids(self):
         mapping = self.get_mapping()
         for sha in self._git.object_store:
@@ -352,25 +351,29 @@ class LocalGitRepository(GitRepository):
                 ret.add(revid)
         return ret
 
-    def _get_parents(self, revid):
+    def _get_parents(self, revid, no_alternates=False):
         if type(revid) != str:
             raise ValueError
         try:
             (hexsha, mapping) = self.lookup_bzr_revision_id(revid)
         except errors.NoSuchRevision:
             return None
+        # FIXME: Honor no_alternates setting
         try:
-            commit = self._git[hexsha]
+            commit = self._git.object_store[hexsha]
         except KeyError:
             return None
         return [
             self.lookup_foreign_revision_id(p, mapping)
             for p in commit.parents]
 
-    def get_parent_map(self, revids):
+    def _get_parent_map_no_fallbacks(self, revids):
+        return self.get_parent_map(revids, no_alternates=True)
+
+    def get_parent_map(self, revids, no_alternates=False):
         parent_map = {}
         for revision_id in revids:
-            parents = self._get_parents(revision_id)
+            parents = self._get_parents(revision_id, no_alternates=no_alternates)
             if revision_id == revision.NULL_REVISION:
                 parent_map[revision_id] = ()
                 continue
@@ -414,15 +417,17 @@ class LocalGitRepository(GitRepository):
     def lookup_foreign_revision_id(self, foreign_revid, mapping=None):
         """Lookup a revision id.
 
+        :param foreign_revid: Foreign revision id to look up
+        :param mapping: Mapping to use (use default mapping if not specified)
+        :raise KeyError: If foreign revision was not found
+        :return: bzr revision id
         """
         assert type(foreign_revid) is str
         if mapping is None:
             mapping = self.get_mapping()
         if foreign_revid == ZERO_SHA:
             return revision.NULL_REVISION
-        commit = self._git.object_store[foreign_revid]
-        while isinstance(commit, Tag):
-            commit = self._git[commit.object[1]]
+        commit = self._git.object_store.peel_sha(foreign_revid)
         if not isinstance(commit, Commit):
             raise NotCommitError(commit.id)
         rev, roundtrip_revid, verifiers = mapping.import_commit(commit,
@@ -459,13 +464,18 @@ class LocalGitRepository(GitRepository):
             except KeyError:
                 # Update refs from Git commit objects
                 # FIXME: Hitting this a lot will be very inefficient...
-                for git_sha, revid, roundtrip_revid in self._iter_revision_ids():
-                    if not roundtrip_revid:
-                        continue
-                    refname = mapping.revid_as_refname(roundtrip_revid)
-                    self._git.refs[refname] = git_sha
-                    if roundtrip_revid == bzr_revid:
-                        return git_sha, mapping
+                pb = ui.ui_factory.nested_progress_bar()
+                try:
+                    for i, (git_sha, revid, roundtrip_revid) in enumerate(self._iter_revision_ids()):
+                        if not roundtrip_revid:
+                            continue
+                        pb.update("resolving revision id", i)
+                        refname = mapping.revid_as_refname(roundtrip_revid)
+                        self._git.refs[refname] = git_sha
+                        if roundtrip_revid == bzr_revid:
+                            return git_sha, mapping
+                finally:
+                    pb.finished()
                 raise errors.NoSuchRevision(self, bzr_revid)
         else:
             return (git_sha, mapping)
@@ -475,7 +485,7 @@ class LocalGitRepository(GitRepository):
             raise errors.InvalidRevisionId(revision_id, self)
         git_commit_id, mapping = self.lookup_bzr_revision_id(revision_id)
         try:
-            commit = self._git[git_commit_id]
+            commit = self._git.object_store[git_commit_id]
         except KeyError:
             raise errors.NoSuchRevision(self, revision_id)
         revision, roundtrip_revid, verifiers = mapping.import_commit(
@@ -522,8 +532,10 @@ class LocalGitRepository(GitRepository):
         raise NotImplementedError(self.get_inventory)
 
     def set_make_working_trees(self, trees):
-        raise errors.UnsupportedOperation(self.set_make_working_trees, self)
-        # TODO: Set bare= in the configuration bug=777065
+        if trees:
+            self._git.get_config().set(("core", ), "bare", "false")
+        else:
+            self._git.get_config().set(("core", ), "bare", "true")
 
     def fetch_objects(self, determine_wants, graph_walker, resolve_ext_ref,
         progress=None):
