@@ -90,7 +90,7 @@ class TransportRefsContainer(RefsContainer):
         keys = set()
         try:
             iter_files = self.transport.clone(base).iter_files_recursive()
-            keys.update(("%s/%s" % (base, urllib.unquote(refname))).strip("/") for 
+            keys.update(("%s/%s" % (base, urllib.unquote(refname))).strip("/") for
                     refname in iter_files if check_ref_format("%s/%s" % (base, refname)))
         except (TransportNotPossible, NoSuchFile):
             pass
@@ -241,8 +241,9 @@ class TransportRefsContainer(RefsContainer):
         :return: True if the set was successful, False otherwise.
         """
         try:
-            realname, _ = self._follow(name)
-        except KeyError:
+            realnames, _ = self.follow(name)
+            realname = realnames[-1]
+        except (KeyError, IndexError):
             realname = name
         self._ensure_dir_exists(realname)
         self.transport.put_bytes(realname, new_ref+"\n")
@@ -259,10 +260,11 @@ class TransportRefsContainer(RefsContainer):
         :return: True if the add was successful, False otherwise.
         """
         try:
-            realname, contents = self._follow(name)
+            realnames, contents = self.follow(name)
             if contents is not None:
                 return False
-        except KeyError:
+            realname = realnames[-1]
+        except (KeyError, IndexError):
             realname = name
         self._check_refname(realname)
         self._ensure_dir_exists(realname)
@@ -318,7 +320,7 @@ class TransportRepo(BaseRepo):
                 refs_container._refs["HEAD"] = head
         else:
             refs_container = TransportRefsContainer(self._controltransport)
-        super(TransportRepo, self).__init__(object_store, 
+        super(TransportRepo, self).__init__(object_store,
                 refs_container)
 
     def get_named_file(self, path):
@@ -411,9 +413,6 @@ class TransportObjectStore(PackBasedObjectStore):
     def __repr__(self):
         return "%s(%r)" % (self.__class__.__name__, self.transport)
 
-    def _pack_cache_stale(self):
-        return False # FIXME
-
     @property
     def alternates(self):
         if self._alternates is not None:
@@ -442,6 +441,17 @@ class TransportObjectStore(PackBasedObjectStore):
         finally:
             f.close()
 
+    @property
+    def packs(self):
+        # FIXME: Never invalidates.
+        if not self._pack_cache:
+            self._update_pack_cache()
+        return self._pack_cache.values()
+
+    def _update_pack_cache(self):
+        for pack in self._load_packs():
+            self._pack_cache[pack._basename] = pack
+
     def _pack_names(self):
         try:
             f = self.transport.get('info/packs')
@@ -475,6 +485,7 @@ class TransportObjectStore(PackBasedObjectStore):
                 idxname = name.replace(".pack", ".idx")
                 idx = load_pack_index_file(idxname, self.pack_transport.get(idxname))
                 pack = Pack.from_objects(pd, idx)
+                pack._basename = idxname[:-5]
                 ret.append(pack)
         return ret
 
@@ -536,7 +547,7 @@ class TransportObjectStore(PackBasedObjectStore):
         idxfile = self.pack_transport.get(basename + ".idx")
         idx = load_pack_index_file(basename+".idx", idxfile)
         final_pack = Pack.from_objects(p, idx)
-        self._add_known_pack(final_pack)
+        self._add_known_pack(basename, final_pack)
         return final_pack
 
     def add_thin_pack(self):
@@ -582,14 +593,15 @@ class TransportObjectStore(PackBasedObjectStore):
             write_pack_index_v2(idxfile, data.sorted_entries(), data_sum)
         finally:
             idxfile.close()
-        final_pack = Pack("pack-%s" % pack_sha)
-        self._add_known_pack(final_pack)
+        basename = "pack-%s" % pack_sha
+        final_pack = Pack(basename)
+        self._add_known_pack(basename, final_pack)
         return final_pack
 
     def add_pack(self):
-        """Add a new pack to this object store. 
+        """Add a new pack to this object store.
 
-        :return: Fileobject to write to and a commit function to 
+        :return: Fileobject to write to and a commit function to
             call when the pack is finished.
         """
         from cStringIO import StringIO
@@ -599,7 +611,9 @@ class TransportObjectStore(PackBasedObjectStore):
                 return self.move_in_pack(f)
             else:
                 return None
-        return f, commit
+        def abort():
+            return None
+        return f, commit, abort
 
     @classmethod
     def init(cls, transport):
