@@ -81,7 +81,7 @@ class GitWorkingTree(workingtree.WorkingTree):
     """A Git working tree."""
 
     def __init__(self, bzrdir, repo, branch, index):
-        self.basedir = bzrdir.root_transport.local_abspath('.')
+        self.basedir = bzrdir.root_transport.local_abspath('.').encode(osutils._fs_enc)
         self.bzrdir = bzrdir
         self.repository = repo
         self.store = self.repository._git.object_store
@@ -201,7 +201,7 @@ class GitWorkingTree(workingtree.WorkingTree):
             try:
                 stat_val = os.lstat(self.abspath(path))
             except (errors.NoSuchFile, OSError):
-                # TODO: Rather than come up with something here, use the 
+                # TODO: Rather than come up with something here, use the
                 # old index
                 stat_val = os.stat_result(
                     (stat.S_IFLNK, 0, 0, 0, 0, 0, 0, 0, 0, 0))
@@ -266,7 +266,7 @@ class GitWorkingTree(workingtree.WorkingTree):
         :param force: Delete files and directories, even if they are changed
             and even if the directories are not empty.
         """
-        all_files = set() # specified and nested files 
+        all_files = set() # specified and nested files
 
         if isinstance(files, basestring):
             files = [files]
@@ -791,7 +791,12 @@ class GitWorkingTree(workingtree.WorkingTree):
         index_mode = entry[-6]
         index_sha = entry[-2]
         disk_path = os.path.join(self.basedir, path)
-        disk_stat = os.lstat(disk_path)
+        try:
+            disk_stat = os.lstat(disk_path)
+        except OSError, (num, msg):
+            if num in (errno.EISDIR, errno.ENOENT):
+                raise KeyError(path)
+            raise
         disk_mtime = disk_stat.st_mtime
         if isinstance(entry[1], tuple):
             index_mtime = entry[1][0]
@@ -859,7 +864,7 @@ class InterIndexGitTree(tree.InterTree):
     @classmethod
     def is_compatible(cls, source, target):
         from bzrlib.plugins.git.repository import GitRevisionTree
-        return (isinstance(source, GitRevisionTree) and 
+        return (isinstance(source, GitRevisionTree) and
                 isinstance(target, GitWorkingTree))
 
     @needs_read_lock
@@ -868,7 +873,7 @@ class InterIndexGitTree(tree.InterTree):
                 want_unversioned=False):
         # FIXME: Handle include_root
         changes = changes_between_git_tree_and_index(
-            self.source.store, self.source.tree, 
+            self.source.store, self.source.tree,
             self.target, want_unchanged=want_unchanged,
             want_unversioned=want_unversioned)
         source_fileid_map = self.source._fileid_map
